/*
 * StreamCluster.java
 *
 * Created on 3. Februar 2003, 11:17
 */


package pipes;
import xxl.core.pipes.operators.AbstractPipe;

/**
 *
 * @author  Jrgen Beringer
 */
public class IDFTStreams extends AbstractPipe<Object[],double[][]>{
	
		
	static boolean DEBUG=false;
	
	//Anzahl von Streams
	int size;
	//Grsse des Gesamten Fensters
	int windowSize;
	//Wurzel der Fenstergrsse (oft bentigt)
	double sqrtWS;
	
	//DFT-Werte der Streams
	double[][] dfts;
	double[] mid;
	//hilfsvariablen
	double[][] values1;
	
	int dftCount;
	
	double faktor;
	
	double[][] output;
	//Beschriftung(Namen) der Streams
	Object[] streamsObjects;
	
	
	public IDFTStreams(Object[] streamsObjects, int size, int windowSize, int dftCount) {
		this(streamsObjects,size,windowSize,dftCount,1f);
	}
	
	/** Creates a new instance of StreamCluster */
	public IDFTStreams(Object[] streamsObjects, int size, int windowSize, int dftCount, double faktor) {
		this.streamsObjects=streamsObjects;
		this.size=size;
		this.dftCount=dftCount;
		this.windowSize=windowSize;
		this.faktor=faktor;
		sqrtWS=(double)Math.sqrt(windowSize);
		//dftweights2=new double[windowSize][2];
		output=new double[size][windowSize];
		
		values1=new double[windowSize][2];
	}
	
	
	public void processObject(Object[] input,int id){
		int aktSize=input.length;
		for (int i=0;i<aktSize;i++){
			dfts=(double[][])input[i];
			for (int m=dftCount+1;m<windowSize-dftCount;m++){
				values1[m][0]=0;
				values1[m][1]=0;
			}
			for (int m=1;m<=dftCount;m++){
				values1[m][0]=dfts[m-1][0];
				values1[m][1]=dfts[m-1][1];
				values1[windowSize-m][0]=dfts[m-1][0];
				values1[windowSize-m][1]=-dfts[m-1][1];
			}
			values1[0][0]=0;
			values1[0][1]=0;
			
			fft(values1);
			
			for (int m=1;m<windowSize;m++){
				double val=Math.sqrt(values1[windowSize-m][0]*values1[windowSize-m][0]+values1[windowSize-m][1]*values1[windowSize-m][1]);
				if (values1[windowSize-m][0]<0.0) val=-val;
				//val+=mid[i];
				output[i][m]=(double)val*faktor;
			}
            output[i][0]=values1[0][0];
		}
		
		
		transfer(output);

	}
	
	static double PI=Math.PI;
	
	static public void fft(double[][] values){
		fftMain(values);
		int n=values.length;
		double sq=Math.sqrt(n);
		for (int i=0;i<n;i++){
			values[i][0]/=sq;
			values[i][1]/=sq;
		}
	}
	
	
	static public void fftMain(double[][] values){
		int n=values.length;
		int nd2=(n>>1);
		int n2,n3;
		int pos2=0;
		double swap0,swap1;
		int k;
		
		for (int i=0;i<n;i++){
			n2=nd2;n3=1;k=0;
			while (n2>=n3){
				if ((i&n2) >0) k=k|n3;
				if ((i&n3) >0) k=k|n2;
				n2=n2>>1;
				n3=n3<<1;
			}
			//System.out.println(k);
			if (i<k){
				//System.out.println("swap :"+i+","+k);
				swap0=values[i][0];
				swap1=values[i][1];
				values[i][0]=values[k][0];
				values[i][1]=values[k][1];
				values[k][0]=swap0;
				values[k][1]=swap1;
			}
		}
		
		int dist=1;
		for (int pot2=2;pot2<=n;pot2<<=1,dist<<=1){
			double u0 = 1.0;
			double u1 = 0.0;
			double w0 =  Math.cos( Math.PI / (double)dist );
			double w1 = -Math.sin( Math.PI / (double)dist );
			for(int j=0;j<dist;j++) {
				for(int i=j;i<n;i+=pot2) {
					pos2=i+dist;
					//System.out.println("pos:"+(i)+" pos2:"+(pos2)+"    "+u0+","+u1);
					double val0=values[pos2][0] * u0 - u1 * values[pos2][1];
					double val1=values[pos2][1] * u0 + u1 * values[pos2][0];
					values[pos2][0] = values[i][0] - val0;
					values[pos2][1] = values[i][1] - val1;
					values[i][0] =  values[i][0] + val0;
					values[i][1] =  values[i][1] + val1;
				}
				double h=u0*w0-w1*u1;
				u1=w0*u1+w1*u0;
				u0=h;
			}
		}
	}
	
}

