/*
 * Created on 24.04.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package pipes;

import xxl.core.pipes.operators.AbstractPipe;
import xxl.core.pipes.operators.identities.BufferPipe;
import clustering.FuzzyKMeansClustering;
import clustering.FuzzyKMeansState;
import clustering.RelativeFuzzyKMeans;

/**
 * @author beringer
 */
public abstract class FuzzyStreamingKMeans extends
        AbstractPipe<Object[], FuzzyKMeansState> {
    static protected boolean DEBUG = false;

    static public boolean QUIET = true;

    protected double lambda = 400;

    protected FuzzyKMeansClustering fkmAlg;

    protected RelativeFuzzyKMeans rfkm;
    
    protected FuzzyKMeansState state;

    

    // Decoupler-Object
    protected BufferPipe bufferPipe;

    // initialisiert? also ob die ersten Cluster schon berechnet wurden
    protected boolean initialized;

    // anzahl von verarbeiteten Blcken
    protected int init;
    
    protected int initCount;

    // Anzahl von unvernderten Neuklassifizierungen in Folge
    protected int zerolength;

    protected boolean maxTest;

    // Beschriftung(Namen) der Streams
    protected String[] streamNames;

    // Number of tested streams as new cluster center
    protected int testCount = 10;

    public FuzzyStreamingKMeans(RelativeFuzzyKMeans rfkm,
            String[] streamNames, int windowSize, int blockCount, int clusterCount) {
        this(rfkm, streamNames, windowSize, blockCount, clusterCount, clusterCount, clusterCount);
    }
    
    public FuzzyStreamingKMeans(RelativeFuzzyKMeans rfkm,
            String[] streamNames, int windowSize, int blockCount,
            int maxClusterCount, int clusterCount) {
        this(rfkm, streamNames, windowSize, blockCount, maxClusterCount, 2, clusterCount);
    }

    /** Creates a new instance of StreamCluster */
    public FuzzyStreamingKMeans(RelativeFuzzyKMeans rfkm,
            String[] streamNames, int windowSize, int blockCount,
            int maxClusterCount,int minClusterCount, int clusterCount) {
        super();

        this.fkmAlg = rfkm.getFuzzyKMeansClustering();
        this.rfkm = rfkm;
        rfkm.setTestCount(testCount);
        this.streamNames = streamNames;
        
        state=new FuzzyKMeansState();
        state.fkmc=fkmAlg;
        state.dataCount=windowSize;
        state.maxClusterCount=maxClusterCount;
        state.minClusterCount=minClusterCount;
        state.clusterCount=clusterCount;
        state.kMeans= new Object[maxClusterCount];
        state.fuzzyCluster = new double[windowSize][maxClusterCount];
        state.clusterSquaredDist = new double[windowSize][maxClusterCount];
        state.clusterCounts = new double[maxClusterCount];
        
        initCount=blockCount;
        initialized = false;
        init = 0;

    }

    public FuzzyKMeansClustering getFKMC() {
        return fkmAlg;
    }

    public void setBufferPipe(BufferPipe decoupler) {
        this.bufferPipe = decoupler;
    }

    public void processObject(Object[] normdfts, int id) {
        if (DEBUG)
            System.out.println("Block " + (init + 1));

        state.data = normdfts;
        init++;
//        if (!QUIET)
//            System.out.println("Step " + init);

        if (initialized) {

            computeClusterCenters();
            computeMemberships();
            computeClusterCenters();
            computeMemberships();
            computeClusterCenters();
            computeMemberships();
            computeClusterCenters();

            FuzzyKMeansState oldState=(FuzzyKMeansState)state.clone();
            oldState.validity=fkmAlg.computeValidity(state);
           
            state=rfkm.testClusterCount(state);

            iteration();
            
            state.validity=fkmAlg.computeValidity(state);
            
            
            if (state.clusterCount!=oldState.clusterCount && state.validity>oldState.validity){
                state=oldState;
                computeClusterCenters();
                state.validity=fkmAlg.computeValidity(state);
                state.objective=fkmAlg.computeObjective(state);
            }
            else computeClusterCenters();
            transfer(state);
            if (DEBUG)
                System.out.println("sep:" + state.validity);
            // if ((init+1)%10==0) showClusters();
            // output2();
        } else {
            if (init == initCount + 1) {
                // int sp=size/clusterCount;
                fkmAlg.initialize(state);

                maxTest = false;
                initialized = true;
                fkmAlg.computeMemberships(state);

                if (!QUIET)
                    System.out.println("Result:1");

                computeClusterCenters();
                computeMemberships();
                computeClusterCenters();
                computeMemberships();
                computeClusterCenters();

                if (DEBUG)
                    System.out.println("initialized");

                int oldClusterCount;
                int iter = 0;
                do {
                    oldClusterCount = state.clusterCount;
                    state=rfkm.testClusterCount(state);
                    iter++;
                } while (oldClusterCount != state.clusterCount && iter < 20);
                iteration();
                computeClusterCenters();
                transfer(state);
            }
        }
    }

    protected void output() {
        for (int i = 0; i < state.dataCount; i++) {
            for (int j = 0; j < state.clusterCount; j++) {
                System.out.println("Stream:" + i + "  Cluster:" + j + "   = "
                        + state.fuzzyCluster[i][j]);
            }
        }
    }
    
    
    protected void computeMemberships() {
        if (maxTest)
            fkmAlg.computeMemberships(state);
        fkmAlg.computeMemberships(state);
    }

    protected void computeClusterCenters() {
        fkmAlg.computeClusterCenters(state);
        if (maxTest)
            fkmAlg.computeClusterCenters(state);
    }

    protected void iteration() {
        int c = 0;
        zerolength = 0;

        do {
            computeMemberships();
            computeClusterCenters();
            c++;
        } while ((bufferPipe==null || bufferPipe.getBufferSize() == 0) && (c < 20)
                && (zerolength < 20));
        if (c < 20)
            System.out.print('.');

        if (DEBUG)
            output();
        if (DEBUG)
            System.out.println("its: " + c + "    " + state.clusterCount);
    }

    protected double computeDistance(Object d1, Object d2) {
        return fkmAlg.getDistance().distance(d1, d2);
    }

    protected Object copyObject(Object d1, Object d2) {
        return fkmAlg.getCopyFunction().invoke(d1, d2);
    }

    public Object[] getClusterMeans() {
        return state.kMeans;
    }

    public int getClusterCount() {
        return state.clusterCount;
    }

    public int getMaxClusterCount() {
        return state.maxClusterCount;
    }

    

}
